/**********************************************************
 SixXS - Automatic IPv6 Connectivity Configuration Utility
***********************************************************
 Copyright 2003-2005 SixXS - http://www.sixxs.net
***********************************************************
 unix-client/aiccu.c - AICCU - The client for UNIX
***********************************************************
 $Author: jeroen $
 $Id: main.c,v 1.5 2005/01/31 19:05:34 jeroen Exp $
 $Date: 2005/01/31 19:05:34 $
**********************************************************/

#include "../common/aiccu.h"

/* 
 * AICCU! - Aka... let's get connected ;)
 * returns a TIC_Tunnel which can then be
 * used for configuring and keeping it running
 */
struct TIC_Tunnel *get_tunnel()
{
	
	struct TIC_sTunnel	*hsTunnel, *t;
	struct TIC_Tunnel	*hTunnel;

	/* Login to the TIC Server */
	if (!tic_Login(g_aiccu->tic, g_aiccu->username, g_aiccu->password)) return false;

	/* 
	 * Don't try to list the tunnels when
	 * we already have a tunnel_id configured
	 */
	if (!g_aiccu->tunnel_id)
	{	
		hsTunnel = tic_ListTunnels(g_aiccu->tic);
		if (!hsTunnel)
		{
			dolog(LOG_ERR, "No tunnel available, request one first\n");
			tic_Free_sTunnel(hsTunnel);
			tic_Logout(g_aiccu->tic, "I didn't have any tunnels to select");
			return NULL;
		}
	
		if (hsTunnel->next)
		{
			dolog(LOG_ERR, "Multiple tunnels available, please pick one from the following list and configure the aiccu.conf using it\n");
			for (t = hsTunnel; t; t = t->next)
			{
				dolog(LOG_ERR, "%s %s %s %s\n", t->sId, t->sIPv6, t->sIPv4, t->sPOPId);
			}
			tic_Free_sTunnel(hsTunnel);
			tic_Logout(g_aiccu->tic, "User still needed to select a tunnel");
			return NULL;
		}
		g_aiccu->tunnel_id = strdup(hsTunnel->sId);

		/* Free the info */
		tic_Free_sTunnel(hsTunnel);
	}

	/* Get Tunnel Information */
	hTunnel = tic_GetTunnel(g_aiccu->tic, g_aiccu->tunnel_id);
	if (!hTunnel)
	{
		tic_Logout(g_aiccu->tic, "No such tunnel");
		return NULL;
	}

	/* Logout, TIC is not needed any more */
	tic_Logout(g_aiccu->tic, NULL);

	/* Swee.... sufficient information */
	return hTunnel;
}

enum AICCU_MODES
{
	A_NONE = 0,
	A_START,
	A_STOP,
	A_TEST,
	A_AUTOTEST,
	A_LICENSE
};

int main(int argc, char *argv[])
{
	enum AICCU_MODES mode = A_NONE;

	struct TIC_Tunnel *hTunnel;
#ifdef _WIN32
	WSADATA wsadata;

	/* Initialize Winsock so that we can do network functions */
	WSAStartup(WINSOCK_VERSION, &wsadata);
#endif

	/* Initialize Configuration */
	aiccu_InitConfig();

	/* Make sure we actually have an IPv6 stack */
	aiccu_install();

	/* Require start/stop/test */
	if (argc == 2 || argc == 3)
	{
		     if (strcasecmp(argv[1], "start")	== 0) mode = A_START;
		else if (strcasecmp(argv[1], "stop")	== 0) mode = A_STOP;
		else if (strcasecmp(argv[1], "test")	== 0) mode = A_TEST;
		else if (strcasecmp(argv[1], "autotest")== 0) mode = A_AUTOTEST;
		else if (strcasecmp(argv[1], "license")	== 0) mode = A_LICENSE;
	}

	/* Optionally we want a second argument: a config file */
	if ((	argc != 2 &&
		argc != 3) ||
		mode == A_NONE)
	{
		dolog(LOG_ERR, "aiccu (start|stop|test|autotest|license) [<configfile>]\n");
		return -1;
	}

	if (	mode == A_LICENSE)
	{
		printf("%s\n", aiccu_license());
		return 0;
	}

#ifndef _WIN32
	/* start or stop? */
	if (	mode != A_TEST &&
		mode != A_AUTOTEST)
	{
		/* Already running? */
		if (sigrunning(mode == A_STOP ? SIGTERM : SIGHUP) == 1)
		{
			dolog(LOG_ERR, "Already running instance HUP'ed, exiting\n");
			return 0;
		}
	}
#endif

	/* Load a configuration file */
	if (!aiccu_LoadConfig(argc == 2 ? AICCU_CONFIG : argv[2]))
	{
		return -1;
	}

	/* Verify required parameters */
	if (!g_aiccu->username || !g_aiccu->password)
	{
		dolog(LOG_ERR, "Required parameters missing, make sure that username and password are given\n");
		aiccu_FreeConfig();
		return false;
	}

	/* Get our tunnel */
	hTunnel = get_tunnel(g_aiccu->tic);
	
	if (!hTunnel)
	{
		dolog(LOG_ERR, "Couldn't retrieve first tunnel for the above reason, aborting\n");
		aiccu_FreeConfig();
		return -1;
	}

	/* 
	 * We now have sufficient information.
	 * Thus we can logout from the TIC server
	 */
	tic_Logout(g_aiccu->tic, NULL);
	g_aiccu->tic = NULL;

	if (g_aiccu->verbose)
	{
		printf("Tunnel Information for %s:\n",hTunnel->sId);
		printf("POP Id      : %s\n", hTunnel->sPOP_Id);
		printf("IPv6 Local  : %s/%u\n", hTunnel->sIPv6_Local,hTunnel->nIPv6_PrefixLength);
		printf("IPv6 Remote : %s/%u\n", hTunnel->sIPv6_POP,hTunnel->nIPv6_PrefixLength);
		printf("Tunnel Type : %s\n", hTunnel->sType);
		printf("Adminstate  : %s\n", hTunnel->sAdminState);
		printf("Userstate   : %s\n", hTunnel->sUserState);
	}

	/* Do the test thing */
	if (	mode == A_TEST ||
		mode == A_AUTOTEST)
	{
		aiccu_test(hTunnel, strcasecmp(argv[1], "autotest") == 0 ? true : false);

		/* exit as all is done */
		tic_Free_Tunnel(hTunnel);
		aiccu_FreeConfig();
		return 0;
	}

#ifndef _WIN32
	if (	mode == A_START &&
		g_aiccu->daemonize != 0)
	{
		/* Daemonize */
		int i = fork();
		if (i < 0)
		{
			fprintf(stderr, "Couldn't fork\n");
			return -1;
		}
		/* Exit the mother fork */
		if (i != 0) return 0;

		/* Child fork */
		setsid();
		/* Cleanup stdin/out/err */
		freopen("/dev/null","r",stdin);
		freopen("/dev/null","w",stdout);
		freopen("/dev/null","w",stderr);

		savepid();
	}

#endif /* !_WIN32 */

	if (mode == A_STOP)
	{
		aiccu_delete(hTunnel);

		/* Free stuff and exit */
		tic_Free_Tunnel(hTunnel);
		aiccu_FreeConfig();
		return 0;
	}

	/* mode == A_START */

	/* 
	 * Setup our tunnel
	 * This also spawns required threads for AYIYA
	 */
	aiccu_setup(hTunnel, true);

	/* We are spawned, now just beat once in a while. */
	while (strcmp(hTunnel->sType, "6in4-heartbeat") == 0)
	{
		dolog(LOG_DEBUG, "Beat()\n");
		heartbeat_beat(hTunnel);
		sleep(hTunnel->nHeartbeat_Interval);
	}
	aiccu_FreeConfig();

	return 0;
}

